/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "sislogger.h"

struct sislog_data {
        int syslog;  /* Turn syslog printing on */
        int verbose; /* Default verbosity level */
        int debug;   /* Level to log to file or syslog at */
        int quiet;   /* incremental quietness */
        int logfd;   /* Used to flock the logfile */
        char msg_prefix[MAX_LOGPREFIX+1]; /* Prefix all messages with this */
};

static int attempted_init = 0;
static struct sislog_data *_logp = NULL;

/**
 * init_log - initializes the logging system
 * @verbose:    three levels of verbosity (1-3) (takes precendence over quiet)
 * @quiet:      three levels of quiet (1-3)
 * @debug:      log this level and lower to syslog and/or logfile
 * @syslog:     turn syslogging on if non-zero
 * @msg_prefix: prefix all messages with this string (31 chars or less)
 * @logfile:    log to this file
 *
 * Returns: zero on success, non-zero on failure
 */
int init_sislog(int verbose, int quiet, int debug, int syslog,
                const char *msg_prefix, const char *logfile)
{
        int error = 0;

        if(_logp == NULL) {
                _logp = malloc(sizeof(*_logp));
                memset(_logp, 0, sizeof(*_logp));
        }

        _logp->syslog=syslog;
        _logp->verbose=verbose;
        _logp->quiet = quiet;
        _logp->debug=debug;

        if(quiet && verbose) {
                fprintf(stderr, "%s:%d - Programming error: quiet and verbose"
                        "both non-zero...Defaulting to verbose setting.\n", 
                        __FILE__, __LINE__);
                _logp->quiet=0;
        }

        if(msg_prefix != NULL) {
                strncpy(_logp->msg_prefix, msg_prefix, MAX_LOGPREFIX);
        }
        else {
                strncpy(_logp->msg_prefix, "", MAX_LOGPREFIX);
        }

        if(logfile != NULL) {
                if((_logp->logfd = open(logfile, O_WRONLY | O_CREAT | O_APPEND,
                                       S_IWUSR | S_IRUSR | S_IRGRP)) != 0) {
                        error = errno;
                        /* Not sure why i'm getting this...*/
                        if(error == EADDRINUSE) {
                                error = 0;
                        }
                }
        }
        else {
                _logp->logfd = -1;
        }

        return error;
}

/**
 * close_log - cleans up a log object
 *
 */
void close_sislog(void)
{
        if(_logp != NULL) {
                if(_logp->logfd != -1)
                        close(_logp->logfd);
                free(_logp);
                _logp = NULL;
        }
        return;
}

/**
 * print_log - prints the log message to various places
 * @level: log level
 * @file: filename the msg is coming from
 * @line: line number the msg is coming from
 * @format: format string
 *
 * Description: prints to various places based on the level and the
 * defaults passed in to the init_log function
 *
 */
void print_sislog(int level, char *file, int line, const char *format, ...)
{
        va_list args;
        char fdbuf[512];

        va_start(args, format);

        /* if the logger isn't initialized yet, init it to sane defaults */
        if(_logp == NULL) {
                /* Only attempt to init if we haven't already tried before */
                if(attempted_init) {
                        return;
                }
                else if (init_sislog(0, 0, 0, 0, NULL, NULL) != 0) {
                        attempted_init = 1;
                        fprintf(stderr, "Error initializing sislog -"
                                 " no messages will be logged.\n");
                        return;
                }
        }

        /* Printouts to stdout and stdlib */
        switch(level) {
        case LOG_DEBUG:
                if(_logp->verbose > 2) {
                        printf("%s:%d %s", file, line, _logp->msg_prefix);
                        vprintf(format, args);
                        fputc('\n', stdout);
                }
                break;
        case LOG_INFO:
                if(_logp->verbose > 1) {
                        printf("%s", _logp->msg_prefix);
                        vprintf(format, args);
                        fputc('\n', stdout);
                }
                break;
        case LOG_NOTICE:
                if(_logp->verbose > 0) {
                        printf("%s", _logp->msg_prefix);
                        vprintf(format, args);
                        fputc('\n', stdout);
                }
                break;
        case LOG_WARN:
                if(_logp->quiet < 1) {
                        printf("%s", _logp->msg_prefix);
                        vprintf(format, args);
                        fputc('\n', stdout);
                }
                break;
        case LOG_ERR:
                if(_logp->quiet < 2) {
                        fprintf(stderr, "%s", _logp->msg_prefix);
                        vfprintf(stderr, format, args);
                        fputc('\n', stderr);
                }
                break;

        case LOG_CRIT:
                if(_logp->quiet < 3) {
                        fprintf(stderr, "%s", _logp->msg_prefix);
                        vfprintf(stderr, format, args);
                        fputc('\n', stderr);
                }
                break;
        default:
                fprintf(stderr, "%s", _logp->msg_prefix);
                vfprintf(stderr, format, args);
                fputc('\n', stderr);
                break;
        }

        if(level > _logp->debug)
                goto end;

        /* logfile prints */
        if(_logp->logfd != -1) {
                if(flock(_logp->logfd, LOCK_EX) == 0) {
                        memset(fdbuf, 0, 512);
                        sprintf(fdbuf, "%s%s:%d ", _logp->msg_prefix,
                                file, line);
                        write(_logp->logfd, fdbuf, sizeof(fdbuf));
                        memset(fdbuf, 0, 512);
                        vsnprintf(fdbuf, 511, format, args);
                        write(_logp->logfd, fdbuf, sizeof(fdbuf));
                        write(_logp->logfd, "\n", 1);
                        flock(_logp->logfd, LOCK_UN);
                }

        }

        /* syslog prints */
        if(_logp->syslog > 0) {
                vsyslog(level, format, args);
        }

 end:
        va_end(args);

}

/*
 * Local variables:
 * c-file-style: "linux"
 * End:
 */
